package org.dasein.media.io;

import java.io.IOException;
import java.io.InputStream;

public class MultiInputStream extends InputStream {
    private boolean     broken    = false;
    private boolean     extended  = false;
    private int         cacheSize = 0;
    private int[]       data      = null;
    private int         index     = 0;
    private InputStream input     = null;

    public MultiInputStream(InputStream is) {
        super();
        input = is;
    }
    
    public void close() throws IOException {
        input.close();
        data = null;
        extended = false;
        broken = false;
    }

    public int read() throws IOException {
        int b;

        if( index >= cacheSize ) {
            if( broken && !extended ) {
                throw new FullBufferException();
            }
            b = input.read();
            if( b == -1 ) {
                return -1;
            }
            cacheSize++;
            if( data == null ) {
                data = new int[1024];
            }
            if( index >= data.length && !broken ) {
                int[] tmp = new int[data.length*2];

                for(int i=0; i<data.length; i++) {
                    tmp[i] = data[i];
                }
                data = tmp;
                if( data.length >= 4000000 ) {
                    broken = true;
                    extended = true;
                }
            }
            if( !broken ) {
                data[index++] = b;
            }
        }
        else {
            b = data[index++];
        }
        return b;
    }

    public void reset() {
        index = 0;
        extended = false;
    }
}
