package org.dasein.util;

import java.util.Collection;

/**
 * Enables rapid development of fast collection filters that potentially map to different
 * collection types.
 * @author George Reese (george.reese@enstratus.com
 * @param <S> the type of objects in the source
 * @param <T> the type of objects in the filtered results
 */
public abstract class Filter<S,T> {
    public Collection<T> filter(Iterable<S> source) {
        final Iterable<S> items = source;
        
        PopulatorThread<T> populator = new PopulatorThread<T>(new JiteratorPopulator<T>() {
            public void populate(Jiterator<T> iterator) {
                for( S item : items ) {
                    T target = test(item);
                    
                    if( target != null ) {
                        iterator.push(target);
                    }
                }
            }
        });
        populator.populate();
        return populator.getResult();
    }
    
    public Collection<T> filter(S[] source) {
        final S[] items = source;
        
        PopulatorThread<T> populator = new PopulatorThread<T>(new JiteratorPopulator<T>() {
            public void populate(Jiterator<T> iterator) {
                for( S item : items ) {
                    T target = test(item);
                    
                    if( target != null ) {
                        iterator.push(target);
                    }
                }
            }
        });
        populator.populate();
        return populator.getResult();
    }
    
    public abstract T test(S item);
}
