package org.dasein.util;

import java.util.Collection;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.log4j.Logger;

public class PopulatorThread<T> implements Runnable {
    static private final Logger logger = Logger.getLogger(PopulatorThread.class);
    
    static private final ExecutorService threadPool = Executors.newCachedThreadPool();
    
    static {
    	try {
    		PoolTerminator.addTerminationHandler(new Callable<Boolean>() {
    			public Boolean call() {
    				threadPool.shutdown();
    				return true;
    			}
    		});
    	}
    	catch( Throwable ignore ) {
    		// this will get thrown when not in a J2EE container
    	}
    }
    
    static public void terminate() {
    	threadPool.shutdown();
    }
    
    private JitCollection<T>      collection = null;
    private Jiterator<T>          iterator   = null;
    private JiteratorPopulator<T> populator  = null;
    
    public PopulatorThread(JiteratorPopulator<T> populator) {
        this.populator = populator;
        iterator = new Jiterator<T>();
        collection = new JitCollection<T>(iterator, "Jiterator Collection Loader");
    }
    
    public Collection<T> getResult() {
        return collection;
    }
    
    public void populate() {
        threadPool.submit(this);
    }
    
    public void run() {
        boolean success = false;
        
        try {
            Thread.currentThread().setName("Jiterator Populator");
            populator.populate(iterator);
            success = true;
        }
        catch( Exception e ) {
            if( logger.isDebugEnabled() ) {
                logger.debug(e);
                e.printStackTrace();
            }
            iterator.setLoadException(e);
        }
        catch( Throwable t ) {
            if( logger.isDebugEnabled() ) {
                logger.debug(t);
                t.printStackTrace();
            }
            iterator.setLoadException(new RuntimeException(t));
        }
        finally {
            if( success ) {
                iterator.complete();
            }
        }
    }
}
