package org.dasein.util;

import java.util.concurrent.Callable;

import org.apache.log4j.Logger;

public class Retry<T> {
    static private final Logger logger = Logger.getLogger(Retry.class);
    
    static public class RetryException extends RuntimeException {
        private static final long serialVersionUID = -7329075046711516092L;

        public RetryException(String msg) {
            super(msg);
        }
    }

    public T retry(int retries, Callable<T> operation) throws Exception {
        return retry(retries, 2, operation);
    }
    
    public T retry(int retries, int baseDelayInSeconds, Callable<T> operation) throws Exception {
        int maxRetries = retries;
        
        if( retries < 1 ) {
            retries = 1;
        }
        while( retries > 0 ) {
            retries--;
            if( retries > 0 ) {
                try {
                    return operation.call();
                }
                catch( Exception e ) {
                    logger.warn(e);
                    try { Thread.sleep((maxRetries-retries) * baseDelayInSeconds * 1000L); }
                    catch( InterruptedException ignore ) { }
                }
            }
            else {
                return operation.call();
            }
        }
        throw new RetryException("Unreachable situation.");
    }
}
